@@ -38,10 +38,21 @@
 --
 module Network.Wai.Handler.Warp (
     -- * Run a Warp server
+    -- | All of these automatically serve the same 'Application' over HTTP\/1,
+    -- HTTP\/1.1, and HTTP\/2.
     run
   , runEnv
   , runSettings
   , runSettingsSocket
+    -- * Run an HTTP\/2-aware server
+    -- | Each of these takes an HTTP\/2-aware application as well as a backup
+    -- 'Application' to be used for HTTP\/1.1 and HTTP\/1 connections.  These
+    -- are only needed if your application needs access to HTTP\/2-specific
+    -- features such as trailers or pushed streams.
+  , runHTTP2
+  , runHTTP2Env
+  , runHTTP2Settings
+  , runHTTP2SettingsSocket
     -- * Settings
   , Settings
   , defaultSettings
@@ -55,6 +66,7 @@ module Network.Wai.Handler.Warp (
   , setTimeout
   , setManager
   , setFdCacheDuration
+  , setFileInfoCacheDuration
   , setBeforeMainLoop
   , setNoParsePath
   , setInstallShutdownHandler
@@ -65,6 +77,8 @@ module Network.Wai.Handler.Warp (
   , setProxyProtocolRequired
   , setProxyProtocolOptional
   , setSlowlorisSize
+  , setHTTP2Disabled
+  , setLogger
     -- ** Getters
   , getPort
   , getHost
@@ -83,6 +97,8 @@ module Network.Wai.Handler.Warp (
   , InvalidRequest (..)
     -- * Utilities
   , pauseTimeout
+  , FileInfo(..)
+  , getFileInfo
     -- * Internal
     -- | The following APIs will be removed in Warp 3.2.0. Please use ones exported from Network.Wai.Handler.Warp.Internal.
 
@@ -124,16 +140,18 @@ module Network.Wai.Handler.Warp (
   , module Network.Wai.Handler.Warp.Timeout
   ) where
 
-import Control.Exception (SomeException)
+import Control.Exception (SomeException, throwIO)
 import Data.ByteString (ByteString)
 import Data.Maybe (fromMaybe)
 import Data.Streaming.Network (HostPreference)
 import qualified Data.Vault.Lazy as Vault
+import qualified Network.HTTP.Types as H
 import Network.Socket (SockAddr)
 import Network.Wai (Request, Response, vault)
 import Network.Wai.Handler.Warp.Buffer
 import Network.Wai.Handler.Warp.Date
 import Network.Wai.Handler.Warp.FdCache
+import Network.Wai.Handler.Warp.FileInfoCache
 import Network.Wai.Handler.Warp.Header
 import Network.Wai.Handler.Warp.Request
 import Network.Wai.Handler.Warp.Response
@@ -207,10 +225,26 @@ setManager x y = y { settingsManager = Just x }
 -- Enabling this cache results in drastic performance improvement for file
 -- transfers.
 --
--- Default value: since 3.0.13, default value is 0, was previously 10
+-- Default value: 0, was previously 10
+--
+-- Since 3.0.13
 setFdCacheDuration :: Int -> Settings -> Settings
 setFdCacheDuration x y = y { settingsFdCacheDuration = x }
 
+-- | Cache duration time of file information in seconds. 0 means that the cache mechanism is not used.
+--
+-- The file information cache is an optimization that is useful for servers dealing with
+-- static files. However, if files are being modified, it can cause incorrect
+-- results in some cases. Therefore, we disable it by default. If you know that
+-- your files will be static or you prefer performance to file consistency,
+-- it's recommended to turn this on; a reasonable value for those cases is 10.
+-- Enabling this cache results in drastic performance improvement for file
+-- transfers.
+--
+-- Default value: 0
+setFileInfoCacheDuration :: Int -> Settings -> Settings
+setFileInfoCacheDuration x y = y { settingsFileInfoCacheDuration = x }
+
 -- | Code to run after the listening socket is ready but before entering
 -- the main event loop. Useful for signaling to tests that they can start
 -- running, or to drop permissions after binding to a restricted port.
@@ -345,6 +379,19 @@ setProxyProtocolOptional y = y { settingsProxyProtocol = ProxyProtocolOptional }
 setSlowlorisSize :: Int -> Settings -> Settings
 setSlowlorisSize x y = y { settingsSlowlorisSize = x }
 
+-- | Disable HTTP2.
+--
+-- Since 3.1.7
+setHTTP2Disabled :: Settings -> Settings
+setHTTP2Disabled y = y { settingsHTTP2Enabled = False }
+
+-- | Setting a log function.
+--
+-- Since 3.X.X
+setLogger :: (Request -> H.Status -> Maybe Integer -> IO ())
+          -> Settings -> Settings
+setLogger lgr y = y { settingsLogger = lgr }
+
 -- | Explicitly pause the slowloris timeout.
 --
 -- This is useful for cases where you partially consume a request body. For
@@ -353,3 +400,22 @@ setSlowlorisSize x y = y { settingsSlowlorisSize = x }
 -- Since 3.0.10
 pauseTimeout :: Request -> IO ()
 pauseTimeout = fromMaybe (return ()) . Vault.lookup pauseTimeoutKey . vault
+
+-- | Getting file information of the target file.
+--
+--   This function first uses a stat(2) or similar system call
+--   to obtain information of the target file, then registers
+--   it into the internal cache.
+--   From the next time, the information is obtained
+--   from the cache. This reduces the overhead to call the system call.
+--   The internal cache is refreshed every duration specified by
+--   'setFileInfoCacheDuration'.
+--
+--   This function throws an 'IO' exception if the information is not
+--   available. For instance, the target file does not exist.
+--   If this function is used an a Request generated by a WAI
+--   backend besides Warp, it also throws an 'IO' exception.
+--
+-- Since 3.1.10
+getFileInfo :: Request -> FilePath -> IO FileInfo
+getFileInfo = fromMaybe (\_ -> throwIO (userError "getFileInfo")) . Vault.lookup getFileInfoKey . vault
