@@ -7,9 +7,11 @@ import Control.Concurrent (forkIO, killThread)
 import qualified Control.Exception as E
 import Control.Monad (when, unless, replicateM_)
 import Data.ByteString (ByteString)
+
 import Network.HTTP2
 import Network.Socket (SockAddr)
-import Network.Wai
+
+import Network.Wai.HTTP2 (HTTP2Application)
 import Network.Wai.Handler.Warp.HTTP2.EncodeFrame
 import Network.Wai.Handler.Warp.HTTP2.Manager
 import Network.Wai.Handler.Warp.HTTP2.Receiver
@@ -22,7 +24,7 @@ import Network.Wai.Handler.Warp.Types
 
 ----------------------------------------------------------------
 
-http2 :: Connection -> InternalInfo -> SockAddr -> Transport -> S.Settings -> (BufSize -> IO ByteString) -> Application -> IO ()
+http2 :: Connection -> InternalInfo -> SockAddr -> Transport -> S.Settings -> (BufSize -> IO ByteString) -> HTTP2Application -> IO ()
 http2 conn ii addr transport settings readN app = do
     checkTLS
     ok <- checkPreface
@@ -36,7 +38,7 @@ http2 conn ii addr transport settings readN app = do
         -- fixme: hard coding: 10
         replicateM_ 10 $ spawnAction mgr
         -- Receiver
-        let mkreq = mkRequest settings addr
+        let mkreq = mkRequest ii settings addr
         tid <- forkIO $ frameReceiver ctx mkreq readN
         -- Sender
         -- frameSender is the main thread because it ensures to send
