@@ -41,22 +41,22 @@ frameReceiver ctx mkreq recvN = loop `E.catch` sendGoaway
       | Just (ConnectionError err msg) <- E.fromException e = do
           csid <- readIORef currentStreamId
           let frame = goawayFrame csid err msg
-          enqueue outputQ (OGoaway frame) highestPriority
+          enqueueControl outputQ 0 $ OGoaway frame
       | otherwise = return ()
 
     sendReset err sid = do
         let frame = resetFrame err sid
-        enqueue outputQ (OFrame frame) highestPriority
+        enqueueControl outputQ 0 $ OFrame frame
 
     loop = do
         hd <- recvN frameHeaderLength
         if BS.null hd then
-            enqueue outputQ OFinish highestPriority
+            enqueueControl outputQ 0 OFinish
           else do
             cont <- processStreamGuardingError $ decodeFrameHeader hd
             when cont loop
 
-    processStreamGuardingError (_, FrameHeader{streamId})
+    processStreamGuardingError (FrameHeaders, FrameHeader{streamId})
       | isResponse streamId = E.throwIO $ ConnectionError ProtocolError "stream id should be odd"
     processStreamGuardingError (FrameUnknown _, FrameHeader{payloadLength}) = do
         mx <- readIORef continued
@@ -92,7 +92,7 @@ frameReceiver ctx mkreq recvN = loop `E.catch` sendGoaway
           control ftyp header pl ctx
       | otherwise = do
           checkContinued
-          strm@Stream{streamState,streamContentLength,streamPriority} <- getStream
+          strm@Stream{streamState,streamContentLength,streamPrecedence} <- getStream
           pl <- recvN payloadLength
           state <- readIORef streamState
           state' <- stream ftyp header pl ctx state strm
@@ -103,7 +103,7 @@ frameReceiver ctx mkreq recvN = loop `E.catch` sendGoaway
                       Just vh -> do
                           when (isJust (vhCL vh) && vhCL vh /= Just 0) $
                               E.throwIO $ StreamError ProtocolError streamId
-                          writeIORef streamPriority pri
+                          writeIORef streamPrecedence $ toPrecedence pri
                           writeIORef streamState HalfClosed
                           let req = mkreq vh (return "")
                           atomically $ writeTQueue inputQ $ Input strm req
@@ -113,7 +113,7 @@ frameReceiver ctx mkreq recvN = loop `E.catch` sendGoaway
                   case validateHeaders hdr of
                       Just vh -> do
                           q <- newTQueueIO
-                          writeIORef streamPriority pri
+                          writeIORef streamPrecedence $ toPrecedence pri
                           writeIORef streamState (Open (Body q))
                           writeIORef streamContentLength $ vhCL vh
                           readQ <- newReadBody q
@@ -161,15 +161,15 @@ frameReceiver ctx mkreq recvN = loop `E.catch` sendGoaway
                              -- idle, so that receiving frames on it is only a
                              -- stream error.
                              consume payloadLength
-                             strm <- newStream streamId 0
+                             strm <- newStream concurrency streamId 0
                              writeIORef (streamState strm) $ Closed $
                                  ResetByMe $ E.toException $
                                      StreamError RefusedStream streamId
                              insert streamTable streamId strm
                              E.throwIO $ StreamError RefusedStream streamId
                      ws <- initialWindowSize <$> readIORef http2settings
-                     newstrm <- newStream streamId (fromIntegral ws)
-                     when (ftyp == FrameHeaders) $ opened ctx newstrm
+                     newstrm <- newStream concurrency streamId (fromIntegral ws)
+                     when (ftyp == FrameHeaders) $ opened newstrm
                      insert streamTable streamId newstrm
                      return newstrm
 
@@ -186,7 +186,7 @@ control FrameSettings header@FrameHeader{flags} bs Context{http2settings, output
     unless (testAck flags) $ do
         modifyIORef http2settings $ \old -> updateSettings old alist
         let frame = settingsFrame setAck []
-        enqueue outputQ (OFrame frame) highestPriority
+        enqueueControl outputQ 0 $ OSettings frame alist
     return True
 
 control FramePing FrameHeader{flags} bs Context{outputQ} =
@@ -194,11 +194,11 @@ control FramePing FrameHeader{flags} bs Context{outputQ} =
         E.throwIO $ ConnectionError ProtocolError "the ack flag of this ping frame must not be set"
       else do
         let frame = pingFrame bs
-        enqueue outputQ (OFrame frame) defaultPriority
+        enqueueControl outputQ 0 $ OFrame frame
         return True
 
 control FrameGoAway _ _ Context{outputQ} = do
-    enqueue outputQ OFinish highestPriority
+    enqueueControl outputQ 0 OFinish
     return False
 
 control FrameWindowUpdate header bs Context{connectionWindow} = do
@@ -246,7 +246,7 @@ stream FrameHeaders header@FrameHeader{flags} bs ctx (Open JustOpened) Stream{st
         let !siz = BS.length frag
         return $ Open $ Continued [frag] siz 1 endOfStream pri
 
-stream FrameHeaders header@FrameHeader{flags} bs _ s@(Open (Body q)) _ = do
+stream FrameHeaders header@FrameHeader{flags} bs _ (Open (Body q)) _ = do
     -- trailer is not supported.
     -- let's read and ignore it.
     HeadersFrame _ _ <- guardIt $ decodeHeadersFrame header bs
@@ -255,7 +255,8 @@ stream FrameHeaders header@FrameHeader{flags} bs _ s@(Open (Body q)) _ = do
         atomically $ writeTQueue q ""
         return HalfClosed
       else
-        return s
+        -- we don't support continuation here.
+        E.throwIO $ ConnectionError ProtocolError "continuation in trailer is not supported"
 
 stream FrameData
        header@FrameHeader{flags,payloadLength,streamId}
@@ -271,7 +272,7 @@ stream FrameData
         let frame1 = windowUpdateFrame 0 payloadLength
             frame2 = windowUpdateFrame streamNumber payloadLength
             frame = frame1 `BS.append` frame2
-        enqueue outputQ (OFrame frame) highestPriority
+        enqueueControl outputQ 0 $ OFrame frame
     atomically $ writeTQueue q body
     if endOfStream then do
         mcl <- readIORef streamContentLength
@@ -310,24 +311,29 @@ stream FrameWindowUpdate header@FrameHeader{streamId} bs _ s Stream{streamWindow
     atomically $ writeTVar streamWindow w
     return s
 
-stream FrameRSTStream header bs ctx _ strm = do
+stream FrameRSTStream header bs _ _ strm = do
     RSTStreamFrame e <- guardIt $ decoderstStreamFrame header bs
     let cc = Reset e
-    closed ctx strm cc
+    closed strm cc
     return $ Closed cc -- will be written to streamState again
 
-stream FramePriority header bs Context{outputQ} s Stream{streamNumber,streamPriority} = do
-    PriorityFrame p <- guardIt $ decodePriorityFrame header bs
-    checkPriority p streamNumber
+stream FramePriority header bs Context{outputQ,priorityTreeSize} s Stream{streamNumber,streamPrecedence} = do
+    PriorityFrame newpri <- guardIt $ decodePriorityFrame header bs
+    checkPriority newpri streamNumber
     -- checkme: this should be tested
-    -- fixme: This works well when the priority gets lower because
-    -- the old higher priority value comes out from the queue quickly
-    -- and the new lower priority is used when enqueuing again.
-    -- But when the priority get higher, it takes time to use the new
-    -- priority.
-    writeIORef streamPriority p
-    -- checkme: this should be tested
-    when (isIdle s) $ prepare outputQ streamNumber p
+    oldpre <- readIORef streamPrecedence
+    let !newpre = toPrecedence newpri
+    writeIORef streamPrecedence newpre
+    if isIdle s then do
+        n <- atomicModifyIORef' priorityTreeSize (\x -> (x+1,x+1))
+        -- fixme hard coding
+        when (n >= 20) $ E.throwIO $ ConnectionError EnhanceYourCalm "too many idle priority frames"
+        prepare outputQ streamNumber newpri
+      else do
+        mx <- delete outputQ streamNumber oldpre
+        case mx of
+            Nothing -> return ()
+            Just x  -> enqueue outputQ streamNumber newpre x
     return s
 
 -- this ordering is important
