@@ -1,3 +1,5 @@
+{-# LANGUAGE CPP #-}
+
 module Network.Wai.Handler.Warp.MultiMap (
     MMap
   , Some(..)
@@ -16,7 +18,9 @@ module Network.Wai.Handler.Warp.MultiMap (
   , merge
   ) where
 
+#if __GLASGOW_HASKELL__ < 709
 import Control.Applicative ((<$>))
+#endif
 import Data.List (foldl')
 
 ----------------------------------------------------------------
@@ -72,7 +76,7 @@ searchWith xk f (Node _ l k v r) = case compare xk k of
 ----------------------------------------------------------------
 
 -- | O(1)
-isEmpty :: (Eq k, Eq v) => MMap k v -> Bool
+isEmpty :: MMap k v -> Bool
 isEmpty Leaf = True
 isEmpty _    = False
 
