@@ -7,6 +7,7 @@ module Network.Wai.Handler.Warp.Request (
     recvRequest
   , headerLines
   , pauseTimeoutKey
+  , getFileInfoKey
   ) where
 
 import qualified Control.Concurrent as Conc (yield)
@@ -21,6 +22,7 @@ import qualified Network.HTTP.Types as H
 import Network.Socket (SockAddr)
 import Network.Wai
 import Network.Wai.Handler.Warp.Conduit
+import Network.Wai.Handler.Warp.FileInfoCache
 import Network.Wai.Handler.Warp.Header
 import Network.Wai.Handler.Warp.ReadInt
 import Network.Wai.Handler.Warp.RequestHeader
@@ -61,9 +63,9 @@ recvRequest settings conn ii addr src = do
     hdrlines <- headerLines src
     (method, unparsedPath, path, query, httpversion, hdr) <- parseHeaderLines hdrlines
     let idxhdr = indexRequestHeader hdr
-        expect = idxhdr ! idxExpect
-        cl = idxhdr ! idxContentLength
-        te = idxhdr ! idxTransferEncoding
+        expect = idxhdr ! fromEnum ReqExpect
+        cl = idxhdr ! fromEnum ReqContentLength
+        te = idxhdr ! fromEnum ReqTransferEncoding
         handle100Continue = handleExpect conn httpversion expect
     (rbody, remainingRef, bodyLength) <- bodyAndSource src cl te
     -- body producing function which will produce '100-continue', if needed
@@ -81,16 +83,17 @@ recvRequest settings conn ii addr src = do
           , isSecure          = False
           , remoteHost        = addr
           , requestBody       = rbody'
-          , vault             = Vault.insert pauseTimeoutKey
-                                (Timeout.pause th)
-                                Vault.empty
+          , vault             = vaultValue
           , requestBodyLength = bodyLength
-          , requestHeaderHost = idxhdr ! idxHost
-          , requestHeaderRange = idxhdr ! idxRange
+          , requestHeaderHost  = idxhdr ! fromEnum ReqHost
+          , requestHeaderRange = idxhdr ! fromEnum ReqRange
           }
     return (req, remainingRef, idxhdr, rbodyFlush)
   where
     th = threadHandle ii
+    vaultValue = Vault.insert pauseTimeoutKey (Timeout.pause th)
+               $ Vault.insert getFileInfoKey (fileInfo ii)
+                 Vault.empty
 
 ----------------------------------------------------------------
 
@@ -278,3 +281,7 @@ checkCR bs pos = if pos > 0 && 13 == S.index bs p then p else pos -- 13 is CR
 pauseTimeoutKey :: Vault.Key (IO ())
 pauseTimeoutKey = unsafePerformIO Vault.newKey
 {-# NOINLINE pauseTimeoutKey #-}
+
+getFileInfoKey :: Vault.Key (FilePath -> IO FileInfo)
+getFileInfoKey = unsafePerformIO Vault.newKey
+{-# NOINLINE getFileInfoKey #-}
