@@ -6,9 +6,8 @@
 
 module Network.Wai.Handler.Warp.Response (
     sendResponse
-  , fileRange -- for testing
+  , sanitizeHeaderValue -- for testing
   , warpVersion
-  , defaultServerValue
   , addDate
   , addServer
   , hasBody
@@ -18,22 +17,26 @@ module Network.Wai.Handler.Warp.Response (
 #define MIN_VERSION_base(x,y,z) 1
 #endif
 
+#ifndef MIN_VERSION_http_types
+#define MIN_VERSION_http_types(x,y,z) 1
+#endif
+
 import Blaze.ByteString.Builder.HTTP (chunkedTransferEncoding, chunkedTransferTerminator)
 #if __GLASGOW_HASKELL__ < 709
 import Control.Applicative
 #endif
-import Control.Exception
+import qualified Control.Exception as E
 import Control.Monad (unless, when)
 import Data.Array ((!))
 import Data.ByteString (ByteString)
 import qualified Data.ByteString as S
+import qualified Data.ByteString.Char8 as S8
 import Data.ByteString.Builder (byteString, Builder)
 import Data.ByteString.Builder.Extra (flush)
-import qualified Data.ByteString.Char8 as B (pack)
 import qualified Data.CaseInsensitive as CI
 import Data.Function (on)
 import Data.List (deleteBy)
-import Data.Maybe (isJust, listToMaybe)
+import Data.Maybe
 #if MIN_VERSION_base(4,5,0)
 # if __GLASGOW_HASKELL__ < 709
 import Data.Monoid (mempty)
@@ -44,21 +47,26 @@ import Data.Monoid (mappend, mempty)
 #endif
 import Data.Streaming.Blaze (newBlazeRecv, reuseBufferStrategy)
 import Data.Version (showVersion)
+import Data.Word8 (_cr, _lf)
 import qualified Network.HTTP.Types as H
+#if MIN_VERSION_http_types(0,9,0)
+import qualified Network.HTTP.Types.Header as H
+#endif
 import Network.Wai
 import Network.Wai.Handler.Warp.Buffer (toBuilderBuffer)
 import qualified Network.Wai.Handler.Warp.Date as D
+#ifndef WINDOWS
 import qualified Network.Wai.Handler.Warp.FdCache as F
+#endif
+import Network.Wai.Handler.Warp.File
 import Network.Wai.Handler.Warp.Header
 import Network.Wai.Handler.Warp.IO (toBufIOWith)
-import Network.Wai.Handler.Warp.RequestHeader (parseByteRanges)
 import Network.Wai.Handler.Warp.ResponseHeader
+import Network.Wai.Handler.Warp.Settings
 import qualified Network.Wai.Handler.Warp.Timeout as T
 import Network.Wai.Handler.Warp.Types
-import Network.Wai.Internal (Response (ResponseBuilder, ResponseStream, ResponseFile, ResponseRaw))
-import Numeric (showInt)
+import Network.Wai.Internal
 import qualified Paths_warp
-import qualified System.PosixCompat.Files as P
 
 #if !MIN_VERSION_base(4,5,0)
 (<>) :: Monoid m => m -> m -> m
@@ -68,74 +76,16 @@ import qualified System.PosixCompat.Files as P
 -- $setup
 -- >>> :set -XOverloadedStrings
 
-mapRight :: (b -> c) -> Either a b -> Either a c
-mapRight f eith = case eith of
-    Right x -> Right (f x)
-    Left l -> Left l
-
-----------------------------------------------------------------
-
-fileRange :: H.Status -> H.ResponseHeaders -> FilePath
-          -> Maybe FilePart -> Maybe HeaderValue
-          -> IO (Either IOException
-                        (H.Status, H.ResponseHeaders, Integer, Integer))
-fileRange s0 hs0 path Nothing mRange =
-    mapRight (fileRangeSized s0 hs0 Nothing mRange . fromIntegral . P.fileSize) <$>
-    try (P.getFileStatus path)
-fileRange s0 hs0 _ mPart@(Just part) mRange =
-    return . Right $ fileRangeSized s0 hs0 mPart mRange size
-  where
-    size = filePartFileSize part
-
-fileRangeSized :: H.Status -> H.ResponseHeaders
-               -> Maybe FilePart -> Maybe HeaderValue -> Integer
-               -> (H.Status, H.ResponseHeaders, Integer, Integer)
-fileRangeSized s0 hs0 mPart mRange fileSize = (s, hs, beg, len)
-  where
-    (beg, end, len, isEntire) = checkPartRange fileSize mPart mRange
-    hs1 = addContentLength len hs0
-    hs | isEntire  = hs1
-       | otherwise = addContentRange beg end fileSize hs1
-    s  | isEntire  = s0
-       | otherwise = H.status206
-
-checkPartRange :: Integer -> Maybe FilePart -> Maybe HeaderValue
-               -> (Integer, Integer, Integer, Bool)
-checkPartRange fileSize = checkPart
-  where
-    checkPart Nothing Nothing = (0, fileSize - 1, fileSize, True)
-    checkPart Nothing (Just range) = case parseByteRanges range >>= listToMaybe of
-        -- Range is broken
-        Nothing              -> (0, fileSize - 1, fileSize, True)
-        Just hrange          -> checkRange hrange
-    -- Ignore Range if FilePart is specified.
-    -- We assume that an application handled Range and specified
-    -- FilePart.
-    checkPart (Just part) _   = (beg, end, len, isEntire)
-      where
-        beg = filePartOffset part
-        len = filePartByteCount part
-        end = beg + len - 1
-        isEntire = beg == 0 && len == fileSize
-
-    checkRange (H.ByteRangeFrom   beg)     = fromRange beg (fileSize - 1)
-    checkRange (H.ByteRangeFromTo beg end) = fromRange beg (min (fileSize - 1) end)
-    checkRange (H.ByteRangeSuffix count)   = fromRange (max 0 (fileSize - count)) (fileSize - 1)
-
-    fromRange beg end = (beg, end, len, isEntire)
-      where
-        len = end - beg + 1
-        isEntire = beg == 0 && len == fileSize
-
 ----------------------------------------------------------------
 
 -- | Sending a HTTP response to 'Connection' according to 'Response'.
 --
---   Applications/middlewares MUST specify a proper 'H.ResponseHeaders'.
+--   Applications/middlewares MUST provide a proper 'H.ResponseHeaders'.
 --   so that inconsistency does not happen.
 --   No header is deleted by this function.
 --
---   Especially, Applications/middlewares MUST take care of
+--   Especially, Applications/middlewares MUST provide a proper
+--   Content-Type. They MUST NOT provide
 --   Content-Length, Content-Range, and Transfer-Encoding
 --   because they are inserted, when necessary,
 --   regardless they already exist.
@@ -147,22 +97,6 @@ checkPartRange fileSize = checkPart
 --
 --   There are three basic APIs to create 'Response':
 --
---   ['responseFile' :: 'H.Status' -> 'H.ResponseHeaders' -> 'FilePath' -> 'Maybe' 'FilePart' -> 'Response']
---     HTTP response body is sent by sendfile() for GET method.
---     HTTP response body is not sent by HEAD method.
---     Applications are categorized into simple and sophisticated.
---     Simple applications should specify 'Nothing' to
---     'Maybe' 'FilePart'. The size of the specified file is obtained
---     by disk access. Then Range is handled.
---     Sophisticated applications should specify 'Just' to
---     'Maybe' 'FilePart'. They should treat Range (and If-Range) by
---     themselves. In both cases,
---     Content-Length and Content-Range (if necessary) are automatically
---     added into the HTTP response header.
---     If Content-Length and Content-Range exist in the HTTP response header,
---     they would cause inconsistency.
---     Status is also changed to 206 (Partial Content) if necessary.
---
 --   ['responseBuilder' :: 'H.Status' -> 'H.ResponseHeaders' -> 'Builder' -> 'Response']
 --     HTTP response body is created from 'Builder'.
 --     Transfer-Encoding: chunked is used in HTTP/1.1.
@@ -173,8 +107,29 @@ checkPartRange fileSize = checkPart
 --
 --   ['responseRaw' :: ('IO' 'ByteString' -> ('ByteString' -> 'IO' ()) -> 'IO' ()) -> 'Response' -> 'Response']
 --     No header is added and no Transfer-Encoding: is applied.
+--
+--   ['responseFile' :: 'H.Status' -> 'H.ResponseHeaders' -> 'FilePath' -> 'Maybe' 'FilePart' -> 'Response']
+--     HTTP response body is sent (by sendfile(), if possible) for GET method.
+--     HTTP response body is not sent by HEAD method.
+--     Content-Length and Content-Range are automatically
+--     added into the HTTP response header if necessary.
+--     If Content-Length and Content-Range exist in the HTTP response header,
+--     they would cause inconsistency.
+--     \"Accept-Ranges: bytes\" is also inserted.
+--
+--     Applications are categorized into simple and sophisticated.
+--     Sophisticated applications should specify 'Just' to
+--     'Maybe' 'FilePart'. They should treat the conditional request
+--     by themselves. A proper 'Status' (200 or 206) must be provided.
+--
+--     Simple applications should specify 'Nothing' to
+--     'Maybe' 'FilePart'. The size of the specified file is obtained
+--     by disk access or from the file infor cache.
+--     If-Modified-Since, If-Unmodified-Since, If-Range and Range
+--     are processed. Since a proper status is chosen, 'Status' is
+--     ignored. Last-Modified is inserted.
 
-sendResponse :: ByteString -- ^ default server value
+sendResponse :: Settings
              -> Connection
              -> InternalInfo
              -> Request -- ^ HTTP request.
@@ -182,35 +137,48 @@ sendResponse :: ByteString -- ^ default server value
              -> IO ByteString -- ^ source from client, for raw response
              -> Response -- ^ HTTP response including status code and response header.
              -> IO Bool -- ^ Returing True if the connection is persistent.
-sendResponse defServer conn ii req reqidxhdr src response = do
+sendResponse settings conn ii req reqidxhdr src response = do
     hs <- addServerAndDate hs0
     if hasBody s then do
-        -- HEAD comes here even if it does not have body.
-        sendRsp conn mfdc ver s hs rsp
+        -- The response to HEAD does not have body.
+        -- But to handle the conditional requests defined RFC 7232 and
+        -- to generate appropriate content-length, content-range,
+        -- and status, the response to HEAD is processed here.
+        --
+        -- See definition of rsp below for proper body stripping.
+        (ms, mlen) <- sendRsp conn ii ver s hs rsp
+        case ms of
+            Nothing         -> return ()
+            Just realStatus -> logger req realStatus mlen
         T.tickle th
         return ret
       else do
-        sendResponseNoBody conn ver s hs
+        _ <- sendRsp conn ii ver s hs RspNoBody
+        logger req s Nothing
         T.tickle th
         return isPersist
   where
+    defServer = settingsServerName settings
+    logger = settingsLogger settings
     ver = httpVersion req
     s = responseStatus response
-    hs0 = responseHeaders response
+    hs0 = sanitizeHeaders $ responseHeaders response
     rspidxhdr = indexResponseHeader hs0
     th = threadHandle ii
     dc = dateCacher ii
-    mfdc = fdCacher ii
     addServerAndDate = addDate dc rspidxhdr . addServer defServer rspidxhdr
-    mRange = reqidxhdr ! idxRange
     (isPersist,isChunked0) = infoFromRequest req reqidxhdr
     isChunked = not isHead && isChunked0
     (isKeepAlive, needsChunked) = infoFromResponse rspidxhdr (isPersist,isChunked)
     isHead = requestMethod req == H.methodHead
     rsp = case response of
-        ResponseFile _ _ path mPart -> RspFile path mPart mRange isHead (T.tickle th)
-        ResponseBuilder _ _ b       -> RspBuilder b needsChunked
-        ResponseStream _ _ fb       -> RspStream fb needsChunked th
+        ResponseFile _ _ path mPart -> RspFile path mPart reqidxhdr isHead (T.tickle th)
+        ResponseBuilder _ _ b
+          | isHead                  -> RspNoBody
+          | otherwise               -> RspBuilder b needsChunked
+        ResponseStream _ _ fb
+          | isHead                  -> RspNoBody
+          | otherwise               -> RspStream fb needsChunked th
         ResponseRaw raw _           -> RspRaw raw src (T.tickle th)
     ret = case response of
         ResponseFile    {} -> isPersist
@@ -220,7 +188,33 @@ sendResponse defServer conn ii req reqidxhdr src response = do
 
 ----------------------------------------------------------------
 
-data Rsp = RspFile FilePath (Maybe FilePart) (Maybe HeaderValue) Bool (IO ())
+sanitizeHeaders :: H.ResponseHeaders -> H.ResponseHeaders
+sanitizeHeaders = map (sanitize <$>)
+  where
+    sanitize v
+      | containsNewlines v = sanitizeHeaderValue v -- slow path
+      | otherwise          = v                     -- fast path
+
+{-# INLINE containsNewlines #-}
+containsNewlines :: ByteString -> Bool
+containsNewlines = S.any (\w -> w == _cr || w == _lf)
+
+{-# INLINE sanitizeHeaderValue #-}
+sanitizeHeaderValue :: ByteString -> ByteString
+sanitizeHeaderValue v = case S8.lines $ S.filter (/= _cr) v of
+    []     -> ""
+    x : xs -> S8.intercalate "\r\n" (x : mapMaybe addSpaceIfMissing xs)
+  where
+    addSpaceIfMissing line = case S8.uncons line of
+        Nothing                           -> Nothing
+        Just (first, _)
+          | first == ' ' || first == '\t' -> Just line
+          | otherwise                     -> Just $ " " <> line
+
+----------------------------------------------------------------
+
+data Rsp = RspNoBody
+         | RspFile FilePath (Maybe FilePart) IndexedHeader Bool (IO ())
          | RspBuilder Builder Bool
          | RspStream StreamingBody Bool T.Handle
          | RspRaw (IO ByteString -> (ByteString -> IO ()) -> IO ()) (IO ByteString) (IO ())
@@ -228,48 +222,20 @@ data Rsp = RspFile FilePath (Maybe FilePart) (Maybe HeaderValue) Bool (IO ())
 ----------------------------------------------------------------
 
 sendRsp :: Connection
-        -> Maybe F.MutableFdCache
+        -> InternalInfo
         -> H.HttpVersion
         -> H.Status
         -> H.ResponseHeaders
         -> Rsp
-        -> IO ()
-sendRsp conn mfdc ver s0 hs0 (RspFile path mPart mRange isHead hook) = do
-    ex <- fileRange s0 hs path mPart mRange
-    case ex of
-        Left _ex ->
-#ifdef WARP_DEBUG
-          print _ex >>
-#endif
-          sendRsp conn mfdc ver s2 hs2 (RspBuilder body True)
-        Right (s, hs1, beg, len)
-          | len >= 0 ->
-            if isHead then
-                sendRsp conn mfdc ver s hs1 (RspBuilder mempty False)
-              else do
-                lheader <- composeHeader ver s hs1
-#ifdef WINDOWS
-                let fid = FileId path Nothing
-                    hook' = hook
-#else
-                (mfd, hook') <- case mfdc of
-                   -- settingsFdCacheDuration is 0
-                   Nothing  -> return (Nothing, hook)
-                   Just fdc -> do
-                      (fd, fresher) <- F.getFd fdc path
-                      return (Just fd, hook >> fresher)
-                let fid = FileId path mfd
-#endif
-                connSendFile conn fid beg len hook' [lheader]
-          | otherwise ->
-            sendRsp conn mfdc ver H.status416
-                (filter (\(k, _) -> k /= "content-length") hs1)
-                (RspBuilder mempty True)
-  where
-    hs = addAcceptRanges hs0
-    s2 = H.status404
-    hs2 =  replaceHeader H.hContentType "text/plain; charset=utf-8" hs0
-    body = byteString "File not found"
+        -> IO (Maybe H.Status, Maybe Integer)
+
+----------------------------------------------------------------
+
+sendRsp conn _ ver s hs RspNoBody = do
+    -- Not adding Content-Length.
+    -- User agents treats it as Content-Length: 0.
+    composeHeader ver s hs >>= connSendAll conn
+    return (Just s, Nothing)
 
 ----------------------------------------------------------------
 
@@ -282,6 +248,7 @@ sendRsp conn _ ver s hs (RspBuilder body needsChunked) = do
         buffer = connWriteBuffer conn
         size = connBufferSize conn
     toBufIOWith buffer size (connSendAll conn) hdrBdy
+    return (Just s, Nothing) -- fixme: can we tell the actual sent bytes?
 
 ----------------------------------------------------------------
 
@@ -305,11 +272,13 @@ sendRsp conn _ ver s hs (RspStream streamingBody needsChunked th) = do
     when needsChunked $ send chunkedTransferTerminator
     mbs <- finish
     maybe (return ()) (sendFragment conn th) mbs
+    return (Just s, Nothing) -- fixme: can we tell the actual sent bytes?
 
 ----------------------------------------------------------------
 
-sendRsp conn _ _ _ _ (RspRaw withApp src tickle) =
+sendRsp conn _ _ _ _ (RspRaw withApp src tickle) = do
     withApp recv send
+    return (Nothing, Nothing)
   where
     recv = do
         bs <- src
@@ -319,12 +288,73 @@ sendRsp conn _ _ _ _ (RspRaw withApp src tickle) =
 
 ----------------------------------------------------------------
 
-sendResponseNoBody :: Connection
-                   -> H.HttpVersion
-                   -> H.Status
-                   -> H.ResponseHeaders
-                   -> IO ()
-sendResponseNoBody conn ver s hs = composeHeader ver s hs >>= connSendAll conn
+-- Sophisticated WAI applications.
+-- We respect s0. s0 MUST be a proper value.
+sendRsp conn ii ver s0 hs0 (RspFile path (Just part) _ isHead hook) =
+    sendRspFile2XX conn ii ver s0 hs path beg len isHead hook
+  where
+    beg = filePartOffset part
+    len = filePartByteCount part
+    hs = addContentHeadersForFilePart hs0 part
+
+----------------------------------------------------------------
+
+-- Simple WAI applications.
+-- Status is ignored
+sendRsp conn ii ver _ hs0 (RspFile path Nothing idxhdr isHead hook) = do
+    efinfo <- E.try $ fileInfo ii path
+    case efinfo of
+        Left (_ex :: E.IOException) ->
+#ifdef WARP_DEBUG
+          print _ex >>
+#endif
+          sendRspFile404 conn ii ver hs0
+        Right finfo -> case conditionalRequest finfo hs0 idxhdr of
+          WithoutBody s         -> sendRsp conn ii ver s hs0 RspNoBody
+          WithBody s hs beg len -> sendRspFile2XX conn ii ver s hs path beg len isHead hook
+
+----------------------------------------------------------------
+
+sendRspFile2XX :: Connection
+               -> InternalInfo
+               -> H.HttpVersion
+               -> H.Status
+               -> H.ResponseHeaders
+               -> FilePath
+               -> Integer
+               -> Integer
+               -> Bool
+               -> IO ()
+               -> IO (Maybe H.Status, Maybe Integer)
+sendRspFile2XX conn ii ver s hs path beg len isHead hook
+  | isHead = sendRsp conn ii ver s hs RspNoBody
+  | otherwise = do
+      lheader <- composeHeader ver s hs
+#ifdef WINDOWS
+      let fid = FileId path Nothing
+          hook' = hook
+#else
+      (mfd, hook') <- case fdCacher ii of
+         -- settingsFdCacheDuration is 0
+         Nothing  -> return (Nothing, hook)
+         Just fdc -> do
+            (fd, fresher) <- F.getFd fdc path
+            return (Just fd, hook >> fresher)
+      let fid = FileId path mfd
+#endif
+      connSendFile conn fid beg len hook' [lheader]
+      return (Just s, Just len)
+
+sendRspFile404 :: Connection
+               -> InternalInfo
+               -> H.HttpVersion
+               -> H.ResponseHeaders
+               -> IO (Maybe H.Status, Maybe Integer)
+sendRspFile404 conn ii ver hs0 = sendRsp conn ii ver s hs (RspBuilder body True)
+  where
+    s = H.notFound404
+    hs =  replaceHeader H.hContentType "text/plain; charset=utf-8" hs0
+    body = byteString "File not found"
 
 ----------------------------------------------------------------
 ----------------------------------------------------------------
@@ -352,7 +382,7 @@ checkPersist req reqidxhdr
     | otherwise       = checkPersist10 conn
   where
     ver = httpVersion req
-    conn = reqidxhdr ! idxConnection
+    conn = reqidxhdr ! fromEnum ReqConnection
     checkPersist11 (Just x)
         | CI.foldCase x == "close"      = False
     checkPersist11 _                    = True
@@ -377,7 +407,7 @@ infoFromResponse rspidxhdr (isPersist,isChunked) = (isKeepAlive, needsChunked)
   where
     needsChunked = isChunked && not hasLength
     isKeepAlive = isPersist && (isChunked || hasLength)
-    hasLength = isJust $ rspidxhdr ! idxContentLength
+    hasLength = isJust $ rspidxhdr ! fromEnum ResContentLength
 
 ----------------------------------------------------------------
 
@@ -390,33 +420,15 @@ hasBody s = sc /= 204
 
 ----------------------------------------------------------------
 
-addAcceptRanges :: H.ResponseHeaders -> H.ResponseHeaders
-addAcceptRanges hdrs = (hAcceptRanges, "bytes") : hdrs
-
 addTransferEncoding :: H.ResponseHeaders -> H.ResponseHeaders
-addTransferEncoding hdrs = (hTransferEncoding, "chunked") : hdrs
-
-addContentLength :: Integer -> H.ResponseHeaders -> H.ResponseHeaders
-addContentLength cl hdrs = (H.hContentLength, len) : hdrs
-  where
-    len = B.pack $ show cl
-
-addContentRange :: Integer -> Integer -> Integer
-                -> H.ResponseHeaders -> H.ResponseHeaders
-addContentRange beg end total hdrs = (hContentRange, range) : hdrs
-  where
-    range = B.pack
-      -- building with ShowS
-      $ 'b' : 'y': 't' : 'e' : 's' : ' '
-      : (if beg > end then ('*':) else
-          showInt beg
-          . ('-' :)
-          . showInt end)
-      ( '/'
-      : showInt total "")
+#if MIN_VERSION_http_types(0,9,0)
+addTransferEncoding hdrs = (H.hTransferEncoding, "chunked") : hdrs
+#else
+addTransferEncoding hdrs = ("transfer-encoding", "chunked") : hdrs
+#endif
 
 addDate :: D.DateCache -> IndexedHeader -> H.ResponseHeaders -> IO H.ResponseHeaders
-addDate dc rspidxhdr hdrs = case rspidxhdr ! idxDate of
+addDate dc rspidxhdr hdrs = case rspidxhdr ! fromEnum ResDate of
     Nothing -> do
         gmtdate <- D.getDate dc
         return $ (H.hDate, gmtdate) : hdrs
@@ -428,12 +440,9 @@ addDate dc rspidxhdr hdrs = case rspidxhdr ! idxDate of
 warpVersion :: String
 warpVersion = showVersion Paths_warp.version
 
-defaultServerValue :: HeaderValue
-defaultServerValue = B.pack $ "Warp/" ++ warpVersion
-
 addServer :: HeaderValue -> IndexedHeader -> H.ResponseHeaders -> H.ResponseHeaders
-addServer defaultServerValue' rspidxhdr hdrs = case rspidxhdr ! idxServer of
-    Nothing -> (hServer, defaultServerValue') : hdrs
+addServer serverName rspidxhdr hdrs = case rspidxhdr ! fromEnum ResServer of
+    Nothing -> (H.hServer, serverName) : hdrs
     _       -> hdrs
 
 ----------------------------------------------------------------
