@@ -2,10 +2,14 @@
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TupleSections #-}
+{-# LANGUAGE PatternGuards #-}
 {-# OPTIONS_GHC -fno-warn-deprecations #-}
 
 module Network.Wai.Handler.Warp.Run where
 
+#if __GLASGOW_HASKELL__ < 709
+import Control.Applicative ((<$>))
+#endif
 import Control.Arrow (first)
 import Control.Concurrent (threadDelay)
 import qualified Control.Concurrent as Conc (yield)
@@ -21,11 +25,13 @@ import Network (sClose, Socket)
 import Network.Socket (accept, withSocketsDo, SockAddr(SockAddrInet, SockAddrInet6))
 import qualified Network.Socket.ByteString as Sock
 import Network.Wai
+import Network.Wai.HTTP2 (HTTP2Application, promoteApplication)
 import Network.Wai.Handler.Warp.Buffer
 import Network.Wai.Handler.Warp.Counter
 import qualified Network.Wai.Handler.Warp.Date as D
 import qualified Network.Wai.Handler.Warp.FdCache as F
-import Network.Wai.Handler.Warp.HTTP2
+import qualified Network.Wai.Handler.Warp.FileInfoCache as I
+import Network.Wai.Handler.Warp.HTTP2 (http2, isHTTP2)
 import Network.Wai.Handler.Warp.Header
 import Network.Wai.Handler.Warp.ReadInt
 import Network.Wai.Handler.Warp.Recv
@@ -68,10 +74,23 @@ allowInterrupt :: IO ()
 allowInterrupt = unblock $ return ()
 #endif
 
+-- Composition over two arguments at once; used for runHTTP2\*.
+(.:) :: (c -> d) -> (a -> b -> c) -> a -> b -> d
+f .: g = curry $ f . uncurry g
+
 -- | Run an 'Application' on the given port.
 -- This calls 'runSettings' with 'defaultSettings'.
 run :: Port -> Application -> IO ()
-run p = runSettings defaultSettings { settingsPort = p }
+run port = runServe port . serveDefault
+
+-- | Serve an 'HTTP2Application' and an 'Application' together on the given
+-- port.
+runHTTP2 :: Port -> HTTP2Application -> Application -> IO ()
+runHTTP2 port = runServe port .: serveHTTP2
+
+-- | The generalized form of 'run'.
+runServe :: Port -> ServeConnection -> IO ()
+runServe p = runServeSettings defaultSettings { settingsPort = p }
 
 -- | Run an 'Application' on the port present in the @PORT@
 -- environment variable. Uses the 'Port' given when the variable is unset.
@@ -79,28 +98,44 @@ run p = runSettings defaultSettings { settingsPort = p }
 --
 -- Since 3.0.9
 runEnv :: Port -> Application -> IO ()
-runEnv p app = do
-    mp <- fmap (lookup "PORT") getEnvironment
+runEnv port = runServeEnv port . serveDefault
 
-    maybe (run p app) runReadPort mp
+-- | The HTTP\/2-aware form of 'runEnv'.
+runHTTP2Env :: Port -> HTTP2Application -> Application -> IO ()
+runHTTP2Env port = runServeEnv port .: serveHTTP2
+
+-- | The generalized form of 'runEnv'.
+runServeEnv :: Port -> ServeConnection -> IO ()
+runServeEnv p serveConn = do
+    mp <- lookup "PORT" <$> getEnvironment
+
+    maybe (runServe p serveConn) runReadPort mp
 
   where
     runReadPort :: String -> IO ()
     runReadPort sp = case reads sp of
-        ((p', _):_) -> run p' app
+        ((p', _):_) -> runServe p' serveConn
         _ -> fail $ "Invalid value in $PORT: " ++ sp
 
 -- | Run an 'Application' with the given 'Settings'.
 -- This opens a listen socket on the port defined in 'Settings' and
 -- calls 'runSettingsSocket'.
 runSettings :: Settings -> Application -> IO ()
-runSettings set app = withSocketsDo $
+runSettings set = runServeSettings set . serveDefault
+
+-- | The HTTP\/2-aware form of 'runSettings'.
+runHTTP2Settings :: Settings -> HTTP2Application -> Application -> IO ()
+runHTTP2Settings set = runServeSettings set .: serveHTTP2
+
+-- | The generalized form of 'runSettings'.
+runServeSettings :: Settings -> ServeConnection -> IO ()
+runServeSettings set serveConn = withSocketsDo $
     bracket
         (bindPortTCP (settingsPort set) (settingsHost set))
         sClose
         (\socket -> do
             setSocketCloseOnExec socket
-            runSettingsSocket set socket app)
+            runServeSettingsSocket set socket serveConn)
 
 -- | This installs a shutdown handler for the given socket and
 -- calls 'runSettingsConnection' with the default connection setup action
@@ -114,9 +149,22 @@ runSettings set app = withSocketsDo $
 -- Note that the 'settingsPort' will still be passed to 'Application's via the
 -- 'serverPort' record.
 runSettingsSocket :: Settings -> Socket -> Application -> IO ()
-runSettingsSocket set socket app = do
+runSettingsSocket set socket = runServeSettingsSocket set socket . serveDefault
+
+-- | The HTTP\/2-aware form of 'runSettingsSocket'.
+runHTTP2SettingsSocket :: Settings
+                       -> Socket
+                       -> HTTP2Application
+                       -> Application
+                       -> IO ()
+runHTTP2SettingsSocket set socket =
+    runServeSettingsSocket set socket .: serveHTTP2
+
+-- | The generalized form of 'runSettingsSocket'.
+runServeSettingsSocket :: Settings -> Socket -> ServeConnection -> IO ()
+runServeSettingsSocket set socket serveConn = do
     settingsInstallShutdownHandler set closeListenSocket
-    runSettingsConnection set getConn app
+    runServeSettingsConnection set getConn serveConn
   where
     getConn = do
 #if WINDOWS
@@ -140,7 +188,16 @@ runSettingsSocket set socket app = do
 --
 -- Since 1.3.5
 runSettingsConnection :: Settings -> IO (Connection, SockAddr) -> Application -> IO ()
-runSettingsConnection set getConn app = runSettingsConnectionMaker set getConnMaker app
+runSettingsConnection set getConn =
+    runServeSettingsConnection set getConn . serveDefault
+
+-- | The generalized form of 'runSettingsConnection'.
+runServeSettingsConnection :: Settings
+                           -> IO (Connection, SockAddr)
+                           -> ServeConnection
+                           -> IO ()
+runServeSettingsConnection set getConn serveConn =
+    runServeSettingsConnectionMaker set getConnMaker serveConn
   where
     getConnMaker = do
       (conn, sa) <- getConn
@@ -149,10 +206,18 @@ runSettingsConnection set getConn app = runSettingsConnectionMaker set getConnMa
 -- | This modifies the connection maker so that it returns 'TCP' for 'Transport'
 -- (i.e. plain HTTP) then calls 'runSettingsConnectionMakerSecure'.
 runSettingsConnectionMaker :: Settings -> IO (IO Connection, SockAddr) -> Application -> IO ()
-runSettingsConnectionMaker x y =
-    runSettingsConnectionMakerSecure x (go y)
+runSettingsConnectionMaker set getConnMaker =
+    runServeSettingsConnectionMaker set getConnMaker . serveDefault
+
+-- | The generalized form of 'runSettingsConnectionMaker'.
+runServeSettingsConnectionMaker :: Settings
+                                -> IO (IO Connection, SockAddr)
+                                -> ServeConnection
+                                -> IO ()
+runServeSettingsConnectionMaker x y =
+    runServeSettingsConnectionMakerSecure x (toTCP <$> y)
   where
-    go = fmap (first (fmap (, TCP)))
+    toTCP = first ((, TCP) <$>)
 
 ----------------------------------------------------------------
 
@@ -163,16 +228,29 @@ runSettingsConnectionMaker x y =
 --
 -- Since 2.1.4
 runSettingsConnectionMakerSecure :: Settings -> IO (IO (Connection, Transport), SockAddr) -> Application -> IO ()
-runSettingsConnectionMakerSecure set getConnMaker app = do
+runSettingsConnectionMakerSecure set getConnMaker =
+    runServeSettingsConnectionMakerSecure set getConnMaker . serveDefault
+
+-- | The generalized form of 'runSettingsConnectionMakerSecure'.
+runServeSettingsConnectionMakerSecure :: Settings
+                                      -> IO (IO (Connection, Transport), SockAddr)
+                                      -> ServeConnection
+                                      -> IO ()
+runServeSettingsConnectionMakerSecure set getConnMaker serveConn = do
     settingsBeforeMainLoop set
     counter <- newCounter
-
-    D.withDateCache $ \dc ->
-        F.withFdCache fdCacheDurationInSeconds $ \fc ->
-            withTimeoutManager $ \tm ->
-                acceptConnection set getConnMaker app dc fc tm counter
+    withII $ acceptConnection set getConnMaker serveConn counter
   where
+    withII action =
+        D.withDateCache $ \dc ->
+        F.withFdCache fdCacheDurationInSeconds $ \fc ->
+        I.withFileInfoCache fdFileInfoDurationInSeconds $ \get ->
+        withTimeoutManager $ \tm -> do
+            let ii0 = InternalInfo undefined tm fc get dc -- fixme: undefined
+            action ii0
+
     fdCacheDurationInSeconds = settingsFdCacheDuration set * 1000000
+    fdFileInfoDurationInSeconds = settingsFileInfoCacheDuration set * 1000000
     withTimeoutManager f = case settingsManager set of
         Just tm -> f tm
         Nothing -> bracket
@@ -195,13 +273,11 @@ runSettingsConnectionMakerSecure set getConnMaker app = do
 -- Our approach is explained in the comments below.
 acceptConnection :: Settings
                  -> IO (IO (Connection, Transport), SockAddr)
-                 -> Application
-                 -> D.DateCache
-                 -> Maybe F.MutableFdCache
-                 -> T.Manager
+                 -> ServeConnection
                  -> Counter
+                 -> InternalInfo
                  -> IO ()
-acceptConnection set getConnMaker app dc fc tm counter = do
+acceptConnection set getConnMaker serveConn counter ii0 = do
     -- First mask all exceptions in acceptLoop. This is necessary to
     -- ensure that no async exception is throw between the call to
     -- acceptNewConnection and the registering of connClose.
@@ -223,7 +299,7 @@ acceptConnection set getConnMaker app dc fc tm counter = do
         case mx of
             Nothing             -> return ()
             Just (mkConn, addr) -> do
-                fork set mkConn addr app dc fc tm counter
+                fork set mkConn addr serveConn counter ii0
                 acceptLoop
 
     acceptNewConnection = do
@@ -247,13 +323,11 @@ acceptConnection set getConnMaker app dc fc tm counter = do
 fork :: Settings
      -> IO (Connection, Transport)
      -> SockAddr
-     -> Application
-     -> D.DateCache
-     -> Maybe F.MutableFdCache
-     -> T.Manager
+     -> ServeConnection
      -> Counter
+     -> InternalInfo
      -> IO ()
-fork set mkConn addr app dc fc tm counter = settingsFork set $ \ unmask ->
+fork set mkConn addr serveConn counter ii0 = settingsFork set $ \ unmask ->
     -- Run the connection maker to get a new connection, and ensure
     -- that the connection is closed. If the mkConn call throws an
     -- exception, we will leak the connection. If the mkConn call is
@@ -268,9 +342,9 @@ fork set mkConn addr app dc fc tm counter = settingsFork set $ \ unmask ->
 
     -- We need to register a timeout handler for this thread, and
     -- cancel that handler as soon as we exit.
-    bracket (T.registerKillThread tm) T.cancel $ \th ->
+    bracket (T.registerKillThread (timeoutManager ii0)) T.cancel $ \th ->
 
-    let ii = InternalInfo th tm fc dc
+    let ii = ii0 { threadHandle = th }
         -- We now have fully registered a connection close handler
         -- in the case of all exceptions, so it is safe to one
         -- again allow async exceptions.
@@ -284,21 +358,29 @@ fork set mkConn addr app dc fc tm counter = settingsFork set $ \ unmask ->
 
        -- Actually serve this connection.
        -- bracket with closeConn above ensures the connection is closed.
-       when goingon $ serveConnection conn ii addr transport set app
+       when goingon $ serveConn conn ii addr transport set
   where
     closeConn (conn, _transport) = connClose conn
 
     onOpen adr    = increase counter >> settingsOnOpen  set adr
     onClose adr _ = decrease counter >> settingsOnClose set adr
 
-serveConnection :: Connection
-                -> InternalInfo
-                -> SockAddr
-                -> Transport
-                -> Settings
-                -> Application
-                -> IO ()
-serveConnection conn ii origAddr transport settings app = do
+-- The type of a function to serve a fully-prepared connection.
+type ServeConnection = Connection
+                    -> InternalInfo
+                    -> SockAddr
+                    -> Transport
+                    -> Settings
+                    -> IO ()
+
+-- Serve an HTTP\/2-unaware Application to a connection over any HTTP version.
+serveDefault :: Application -> ServeConnection
+serveDefault app = serveHTTP2 (promoteApplication app) app
+
+-- Serve an HTTP\/2-aware application over HTTP\/2 or a backup 'Application'
+-- over HTTP\/1.1 or HTTP\/1.
+serveHTTP2 :: HTTP2Application -> Application -> ServeConnection
+serveHTTP2 app2 app conn ii origAddr transport settings = do
     -- fixme: Upgrading to HTTP/2 should be supported.
     (h2,bs) <- if isHTTP2 transport then
                    return (True, "")
@@ -308,10 +390,10 @@ serveConnection conn ii origAddr transport settings app = do
                        return (True, bs0)
                      else
                        return (False, bs0)
-    if h2 then do
+    if settingsHTTP2Enabled settings && h2 then do
         recvN <- makeReceiveN bs (connRecv conn) (connRecvBuf conn)
         -- fixme: origAddr
-        http2 conn ii origAddr transport settings recvN app
+        http2 conn ii origAddr transport settings recvN app2
       else do
         istatus <- newIORef False
         src <- mkSource (wrappedRecv conn th istatus (settingsSlowlorisSize settings))
@@ -364,12 +446,14 @@ serveConnection conn ii origAddr transport settings app = do
 
     th = threadHandle ii
 
+    shouldSendErrorResponse se
+        | Just ConnectionClosedByPeer <- fromException se = False
+        | otherwise                                       = True
+
     sendErrorResponse addr istatus e = do
         status <- readIORef istatus
-        when status $ void $
-            sendResponse
-                (settingsServerName settings)
-                conn ii (dummyreq addr) defaultIndexRequestHeader (return S.empty) (errorResponse e)
+        when (shouldSendErrorResponse e && status) $ void $
+            sendResponse settings conn ii (dummyreq addr) defaultIndexRequestHeader (return S.empty) (errorResponse e)
 
     dummyreq addr = defaultRequest { remoteHost = addr }
 
@@ -401,9 +485,7 @@ serveConnection conn ii origAddr transport settings app = do
             -- send more meaningful error messages to the user.
             -- However, it may affect performance.
             writeIORef istatus False
-            keepAlive <- sendResponse
-                (settingsServerName settings)
-                conn ii req idxhdr (readSource src) res
+            keepAlive <- sendResponse settings conn ii req idxhdr (readSource src) res
             writeIORef keepAliveRef keepAlive
             return ResponseReceived
         keepAlive <- readIORef keepAliveRef
