@@ -48,6 +48,7 @@ data Settings = Settings
     , settingsTimeout :: Int -- ^ Timeout value in seconds. Default value: 30
     , settingsManager :: Maybe Manager -- ^ Use an existing timeout manager instead of spawning a new one. If used, 'settingsTimeout' is ignored. Default is 'Nothing'
     , settingsFdCacheDuration :: Int -- ^ Cache duration time of file descriptors in seconds. 0 means that the cache mechanism is not used. Default value: 0
+    , settingsFileInfoCacheDuration :: Int -- ^ Cache duration time of file information in seconds. 0 means that the cache mechanism is not used. Default value: 0
     , settingsBeforeMainLoop :: IO ()
       -- ^ Code to run after the listening socket is ready but before entering
       -- the main event loop. Useful for signaling to tests that they can start
@@ -92,6 +93,14 @@ data Settings = Settings
       -- ^ Size of bytes read to prevent Slowloris protection. Default value: 2048
       --
       -- Since 3.1.2.
+    , settingsHTTP2Enabled :: Bool
+      -- ^ Whether to enable HTTP2 ALPN/upgrades. Default: True
+      --
+      -- Since 3.1.7.
+    , settingsLogger :: Request -> H.Status -> Maybe Integer -> IO ()
+      -- ^ A log function. Default: no action.
+      --
+      -- Since 3.X.X.
     }
 
 -- | Specify usage of the PROXY protocol.
@@ -115,6 +124,7 @@ defaultSettings = Settings
     , settingsTimeout = 30
     , settingsManager = Nothing
     , settingsFdCacheDuration = 0
+    , settingsFileInfoCacheDuration = 0
     , settingsBeforeMainLoop = return ()
     , settingsFork = void . forkIOWithUnmask
     , settingsNoParsePath = False
@@ -123,6 +133,8 @@ defaultSettings = Settings
     , settingsMaximumBodyFlush = Just 8192
     , settingsProxyProtocol = ProxyProtocolNone
     , settingsSlowlorisSize = 2048
+    , settingsHTTP2Enabled = True
+    , settingsLogger = \_ _ _ -> return ()
     }
 
 -- | Apply the logic provided by 'defaultOnException' to determine if an
