@@ -11,9 +11,9 @@ import Data.IORef (IORef, readIORef, writeIORef, newIORef)
 import Data.Typeable (Typeable)
 import Data.Word (Word16, Word8)
 import Foreign.Ptr (Ptr)
-import Network.HTTP.Types.Header
 import qualified Network.Wai.Handler.Warp.Date as D
 import qualified Network.Wai.Handler.Warp.FdCache as F
+import qualified Network.Wai.Handler.Warp.FileInfoCache as I
 import qualified Network.Wai.Handler.Warp.Timeout as T
 
 #ifndef WINDOWS
@@ -30,18 +30,6 @@ type Port = Int
 -- | The type for header value used with 'HeaderName'.
 type HeaderValue = ByteString
 
-hTransferEncoding :: HeaderName
-hTransferEncoding = "Transfer-Encoding"
-
-hContentRange :: HeaderName
-hContentRange = "Content-Range"
-
-hAcceptRanges :: HeaderName
-hAcceptRanges = "Accept-Ranges"
-
-hServer :: HeaderName
-hServer = "Server"
-
 ----------------------------------------------------------------
 
 -- | Error types for bad 'Request'.
@@ -131,6 +119,7 @@ data InternalInfo = InternalInfo {
     threadHandle :: T.Handle
   , timeoutManager :: T.Manager
   , fdCacher :: Maybe F.MutableFdCache
+  , fileInfo :: FilePath -> IO I.FileInfo
   , dateCacher :: D.DateCache
   }
 
