@@ -18,7 +18,6 @@ import qualified Data.ByteString.Lazy as L
 import qualified Data.IORef as I
 import Data.Streaming.Network (bindPortTCP, getSocketTCP, safeRecv)
 import Network (connectTo, PortID (PortNumber))
-import qualified Network.HTTP as HTTP
 import Network.HTTP.Types
 import Network.Socket (sClose)
 import Network.Socket.ByteString (sendAll)
@@ -29,6 +28,8 @@ import System.IO.Unsafe (unsafePerformIO)
 import System.Timeout (timeout)
 import Test.Hspec
 
+import HTTP
+
 main :: IO ()
 main = hspec spec
 
@@ -349,11 +350,9 @@ spec = do
                 , ("date", "date")
                 ] ""
         withApp defaultSettings app $ \port -> do
-            Right res <- HTTP.simpleHTTP (HTTP.getRequest $ "http://127.0.0.1:" ++ show port)
-            map HTTP.hdrValue (HTTP.retrieveHeaders HTTP.HdrServer res)
-                `shouldBe` ["server"]
-            map HTTP.hdrValue (HTTP.retrieveHeaders HTTP.HdrDate res)
-                `shouldBe` ["date"]
+            res <- sendGET $ "http://127.0.0.1:" ++ show port
+            getHeaderValue HdrServer res `shouldBe` Just "server"
+            getHeaderValue HdrDate res `shouldBe` Just "date"
 
     it "streaming echo #249" $ do
         let app req f = f $ responseStream status200 [] $ \write _ -> do
@@ -375,8 +374,33 @@ spec = do
         let app _ f = f $ responseStream status200 [("content-length", "20")] $ \write _ -> do
                 replicateM_ 4 $ write $ byteString "Hello"
         withApp defaultSettings app $ \port -> do
-            Right res <- HTTP.simpleHTTP (HTTP.getRequest $ "http://127.0.0.1:" ++ show port)
-            HTTP.rspBody res `shouldBe` "HelloHelloHelloHello"
+            res <- sendGET $ "http://127.0.0.1:" ++ show port
+            rspBody res `shouldBe` "HelloHelloHelloHello"
+
+    describe "head requests" $ do
+        let fp = "test/head-response"
+        let app req f =
+                f $ case pathInfo req of
+                    ["builder"] -> responseBuilder status200 [] $ error "should never be evaluated"
+                    ["streaming"] -> responseStream status200 [] $ \write _ ->
+                        write $ error "should never be evaluated"
+                    ["file"] -> responseFile status200 [] fp Nothing
+                    _ -> error "invalid path"
+        it "builder" $ withApp defaultSettings app $ \port -> do
+            res <- sendHEAD $ concat ["http://127.0.0.1:", show port, "/builder"]
+            rspBody res `shouldBe` ""
+        it "streaming" $ withApp defaultSettings app $ \port -> do
+            res <- sendHEAD $ concat ["http://127.0.0.1:", show port, "/streaming"]
+            rspBody res `shouldBe` ""
+        it "file, no range" $ withApp defaultSettings app $ \port -> do
+            bs <- S.readFile fp
+            res <- sendHEAD $ concat ["http://127.0.0.1:", show port, "/file"]
+            getHeaderValue HdrContentLength res `shouldBe` Just (show $ S.length bs)
+        it "file, with range" $ withApp defaultSettings app $ \port -> do
+            res <- sendHEADwH
+                (concat ["http://127.0.0.1:", show port, "/file"])
+                [mkHeader HdrRange "bytes=0-1"]
+            getHeaderValue HdrContentLength res `shouldBe` Just "2"
 
 consumeBody :: IO ByteString -> IO [ByteString]
 consumeBody body =
